/*
 * a function for easier namespace use.
 */
function namespace(namespaceString) {
    var parts = namespaceString.split('.'),
        parent = window,
        currentPart = '';

    for(var i = 0, length = parts.length; i < length; i++) {
        currentPart = parts[i];
        parent[currentPart] = parent[currentPart] || {};
        parent = parent[currentPart];
    }

    return parent;
}

/*
 * add shuffle method to array.
 */
Array.prototype.shuffle = function (){
	for(var rnd, tmp, i=this.length; i; rnd=parseInt(Math.random()*i), tmp=this[--i], this[i]=this[rnd], this[rnd]=tmp);
};

/*
 * a basic card deck.
 */
var deck = namespace('SoftwareTalk.CardGames.BlackJack.Deck');

/*
 * the 4 possible suits.
 */
deck.ColorEnum = {
  CLUBS : {value: 1, name: "Clubs"},
  SPADES : {value: 2, name: "Spades"},
  HEARTS : {value: 4, name: "Hearts"},
  DIAMONDS: {value: 3, name: "Diamonds"}
};

/*
 * one card, having a color and a value.
 * value: int, allowed values: 2-15 (for  2-10, Jack, Queen, King, Ace, Joker)
 * color: deck.ColorEnum
 */
deck.Card = function (value, color) {
   this.hidden = true; // whether or not other players can see this card (or AIs can use it to evalute, etc.)
   this.value = value;
   this.color = color;
};
deck.Card.prototype.hideCard = function () {
    this.hidden = true;
}
deck.Card.prototype.showCard = function () {
    this.hidden = false;
}
deck.Card.prototype.isHidden = function () {
    return this.hidden;
}
/*
 * checks if this card has a greater total value than the given card.
 * this is the case if:
 * - the given card is not a card
 * - the value of the given card is smaller than the value of this card
 * - the value of the given card is equal to the value of this card, but its color is equal or smaller
 *
 * so if the cards are equal in total value this will return false.
 */
deck.Card.prototype.isBiggerThan = function (card) {
    if (card == null || typeof(card) !== typeof(this)) {
        return true;
    } else if (this.value > card.value) {
        return true;
    } else if (this.value < card.value) {
        return false;
    } else {
        return this.color.value > card.color.value;
    }
}
deck.Card.prototype.toString = function () {
    var v = "";
    if (this.value >= 2 && this.value <= 10) {
        v = this.value;
    } else {
        switch(this.value) {
            case 11:
                v = "Jack";
                break;
            case 12:
                v = "Queen";
                break;
            case 13:
                v = "King";
                break;
            case 14:
                v = "Ace";
                break;
            case 15:
                v = "Joker";
                break;
            default:
                v = "Invalid";
        }
    }
    if (this.hidden) {
        return "Hidden";
    } else {
        return "" + v + " of " + this.color.name;
    }
};

/*
 * a deck.
 * empty: boolean, if false a deck with 52 cards (alreay shuffled) will be
 *  created. otherwise an empty Deck will be created.
 */
deck.Deck = function(fill) {
    this.cards = new Array();    
    if (fill) {
        this.init();
        this.shuffle();
    }
}
/*
 * fills this deck with exactly 52 cards, sorted.
 * cards: 2-10, jack, queen, king, ace, all 4 colors, no jokers.
 */
deck.Deck.prototype.init = function () {
    for (var color_name in deck.ColorEnum) {
        var color = deck.ColorEnum[color_name];
        for (var value = 2; value <= 14; value++) {
            this.cards.push(new deck.Card(value, color));
        }
    }
}
/*
 * shuffles this deck.
 */
deck.Deck.prototype.shuffle = function () {
    this.cards.shuffle();
}
/*
 * returns the current number of cards in the deck.
 */
deck.Deck.prototype.cardsInDeck = function () {
    return this.cards.length;
}
/*
 * returns an array containing all cards in this deck.
 */
deck.Deck.prototype.getAllCards = function () {
    return this.cards;
}
/*
 *returns the top card of this deck without removing it from the deck.
 *returns null if no more cards are in the deck.
 */
deck.Deck.prototype.peekCard = function () {
    if (this.cardsInDeck() <= 0) {
        return null;
    }
    return this.cards[0];
}
/*
 * returns an array containing the top n cards or null if less than the given
 * amount of cards are left in the deck.
 * the returned cards will NOT be removed from the deck.
 */
deck.Deck.prototype.peekCards = function (amount) {
    if (this.cardsInDeck() < amount) {
        return null;
    } else {
        return this.cards.slice(0, amount);
    }
}
/*
 *returns the top card of this deck and removes it from the deck.
 *returns null if no more cards are in the deck.
 */
deck.Deck.prototype.getCard = function () {
    if (this.cardsInDeck() <= 0) {
        return null;
    }
    return this.cards.shift();
}
/*
 * returns an array containing the top n cards or null if less than the given
 * amount of cards are left in the deck.
 * the returned cards will be removed from this deck.
 */
deck.Deck.prototype.getCards = function (amount) {
    if (this.cardsInDeck() < amount) {
        return null;
    } else {
        return this.cards.splice(0, amount);
    }
}
/*
 * adds the given card to the bottom of this deck.
 */
deck.Deck.prototype.putCard = function (card) {
    return this.cards.push(card);
};
/*
 * adds the given array of cards to the bottom of this deck.
 */
deck.Deck.prototype.putCards = function (cards) {
    this.cards = this.cards.concat(cards);
};
/*
 * adds a joker to the bottom of this deck of cards.
 * the joker will have the color hearts.
 */
deck.Deck.prototype.putJoker = function () {
    return this.putCard(new deck.Card(15, deck.ColorEnum.HEARTS));
};
deck.Deck.prototype.toString = function () {
    return this.cards.join(', ');
};

deck.getMultiDeck = function(numberofDecks) {
    var mydeck = new deck.Deck(true);
    for (var i = 1; i < numberofDecks; i++) {
        mydeck.putCards((new deck.Deck(true)).getAllCards());
    }
    mydeck.shuffle();
    return mydeck;
}


// test
/*
var card = new deck.Card(5, deck.ColorEnum.DIAMONDS);
card.info();
var card2 = new deck.Card(5, deck.ColorEnum.DIAMONDS);
card2.info();j

var is = "";
if (card.isBiggerThan(card2)) {
    is = "bigger";
} else {
    is = "smaller";
}
alert(card.info() + ' is ' + is + ' than ' + card2.info());
*/

//var deck2 = new deck.Deck();
/*
document.write('<br />' + deck2.cardsInDeck());
for (var i = 0; i < 55; i++) {
document.write('<br />' + deck2.getCard());
document.write('<br />' + deck2.cardsInDeck());
}
*/
/*
document.write('<br />' + deck2.cardsInDeck());
document.write('<br />' + deck2.peek());
document.write('<br />' + deck2.cardsInDeck());
document.write('<br />' + deck2.peek());
*/
/*
document.write('<br />' + deck2);
document.write('<br /><br /><br />' + deck2.cardsInDeck());
document.write('<br />' + deck2.putJoker());
document.write('<br />' + deck2.cardsInDeck());
document.write('<br /><br /><br />' + deck2);
*/
